/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef EXTENSIONS_GENERIC_LOOPS_H
#define EXTENSIONS_GENERIC_LOOPS_H

#include <tuple>

#define _GET_ std::get
#define _GET_TYPE_ std::tuple_element
#define _LENGTH_ std::tuple_size

namespace AMDiS { namespace extensions {
  
namespace tools {
  
  
  // for-loop over sequence of values
  template<typename Seq, long I, long N, class F>
  struct FOR_SUBSEQ
  {
    static void loop() {
      F::call(); // your implementation here
      FOR_SUBSEQ<Seq, I+1, N, F>::loop();
    }
    static void loop_idx() {
      F::call(I); // your implementation here
      FOR_SUBSEQ<Seq, I+1, N, F>::loop_idx();
    }
    static void loop(Seq& s) {
      F::call(_GET_<I>(s)); // your implementation here
      FOR_SUBSEQ<Seq, I+1, N, F>::loop(s);
    }
    static void loop_idx(Seq& s) {
      F::call(I,_GET_<I>(s)); // your implementation here
      FOR_SUBSEQ<Seq, I+1, N, F>::loop_idx(s);
    }
    
    template<typename T>
    static void loop1(Seq& s, T& t) {
      F::call(_GET_<I>(s), t); // your implementation here
      FOR_SUBSEQ<Seq, I+1, N, F>::loop1(s, t);
    }
    template<typename T>
    static void loop1_idx(Seq& s, T& t) {
      F::call(I, _GET_<I>(s), t); // your implementation here
      FOR_SUBSEQ<Seq, I+1, N, F>::loop1_idx(s, t);
    }
    template<typename T1, typename T2>
    static void loop1(Seq& s, T1& t1, T2& t2) {
      F::call(_GET_<I>(s), t1, t2); // your implementation here
      FOR_SUBSEQ<Seq, I+1, N, F>::loop1(s, t1, t2);
    }
    template<typename T1, typename T2>
    static void loop1_idx(Seq& s, T1& t1, T2& t2) {
      F::call(I, _GET_<I>(s), t1, t2); // your implementation here
      FOR_SUBSEQ<Seq, I+1, N, F>::loop1_idx(s, t1, t2);
    }
    
    template<typename T>
    static void loop2(T& t, Seq& s) {
      F::call(t, _GET_<I>(s));
      FOR_SUBSEQ<Seq, I+1, N, F>::loop2(t, s);
    }
    template<typename T>
    static void loop2_idx(T& t, Seq& s) {
      F::template call<I>(t, _GET_<I>(s));
      FOR_SUBSEQ<Seq, I+1, N, F>::loop2_idx(t, s);
    }
    template<typename T0, typename T2>
    static void loop2(T0& t0, Seq& s, T2& t2) {
      F::call(t0, _GET_<I>(s), t2);
      FOR_SUBSEQ<Seq, I+1, N, F>::loop2(t0, s, t2);
    }
    template<typename T0, typename T2>
    static void loop2_idx(T0& t0, Seq& s, T2& t2) {
      F::template call<I>(t0, _GET_<I>(s), t2);
      FOR_SUBSEQ<Seq, I+1, N, F>::loop2_idx(t0, s, t2);
    }
    
    template<typename T0, typename T1>
    static void loop3(T0& t0, T1& t1, Seq& s) {
      F::call(t0, t1, _GET_<I>(s));
      FOR_SUBSEQ<Seq, I+1, N, F>::loop3(t0, t1, s);
    }
    template<typename T0, typename T1>
    static void loop3_idx(T0& t0, T1& t1, Seq& s) {
      F::call(I, t0, t1, _GET_<I>(s));
      FOR_SUBSEQ<Seq, I+1, N, F>::loop3_idx(t0, t1, s);
    }
  };

  // initial/break condition for 'for-loop' over sequences: I==N
  template<typename Seq, long N, class F> 
  struct FOR_SUBSEQ<Seq,N,N,F> 
  { 
    static void loop() {} 
    static void loop_idx() {} 
    static void loop(Seq& t) {}
    static void loop_idx(Seq& t) {}
    
    template<typename T>
    static void loop1(Seq& s, T& t) {}
    template<typename T>
    static void loop1_idx(Seq& s, T& t) {}    
    template<typename T1, typename T2>
    static void loop1(Seq& s, T1& t1, T2& t2) {}
    template<typename T1, typename T2>
    static void loop1_idx(Seq& s, T1& t1, T2& t2) {}
    
    template<typename T>
    static void loop2(T& t, Seq& s) {}
    template<typename T>
    static void loop2_idx(T& t, Seq& s) {}
    template<typename T0, typename T2>
    static void loop2(T0& t0, Seq& s, T2& t2) {}
    template<typename T0, typename T2>
    static void loop2_idx(T0& t0, Seq& s, T2& t2) {}
    
    template<typename T0, typename T1>
    static void loop3(T0& t0, T1& t1, Seq& s) {}
    template<typename T0, typename T1>
    static void loop3_idx(T0& t0, T1& t1, Seq& s) {}
  };

  /// \brief
  /// interface for loop over sequence of values
  /// for i in {seq[0]..seq[end]}
  ///
  template<class F> 
  struct FOR_EACH
  {
    template<typename Seq>
    static void loop() { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop(); } 
    template<typename Seq>
    static void loop_idx() { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop_idx(); } 
    template<typename Seq>
    static void loop(Seq& s) { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop(s); } 
    template<typename Seq>
    static void loop_idx(Seq& s) { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop_idx(s); } 
    
    template<typename Seq, typename T>
    static void loop1(Seq& s, T& t) { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop1(s, t); } 
    template<typename Seq, typename T>
    static void loop1_idx(Seq& s, T& t) { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop1_idx(s, t); } 
    template<typename Seq, typename T1, typename T2>
    static void loop1(Seq& s, T1& t1, T2& t2) { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop1(s, t1, t2); } 
    template<typename Seq, typename T1, typename T2>
    static void loop1_idx(Seq& s, T1& t1, T2& t2) { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop1_idx(s, t1, t2); } 
    
    template<typename T, typename Seq>
    static void loop2(T& t, Seq& s) { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop2(t, s); } 
    template<typename T, typename Seq>
    static void loop2_idx(T& t, Seq& s) { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop2_idx(t, s); } 
    template<typename T0, typename Seq, typename T2>
    static void loop2(T0& t0, Seq& s, T2& t2) { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop2(t0, s, t2); } 
    template<typename T0, typename Seq, typename T2>
    static void loop2_idx(T0& t0, Seq& s, T2& t2) { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop2_idx(t0, s, t2); } 
    
    template<typename T0, typename T1, typename Seq>
    static void loop3(T0& t0, T1& t1, Seq& s) { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop3(t0, t1, s); } 
    template<typename T0, typename T1, typename Seq>
    static void loop3_idx(T0& t0, T1& t1, Seq& s) { FOR_SUBSEQ<Seq, 0, _LENGTH_<Seq>::value, F>::loop3_idx(t0, t1, s); } 
  };
  
} // namespace tools

} }

#endif // EXTENSIONS_GENERIC_LOOPS_H