/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef CAHN_HILLIARD_H
#define CAHN_HILLIARD_H

#include "AMDiS.h"
#include "BaseProblem.h"
#include "ExtendedProblemStat.h"
#include "HL_SignedDistTraverse.h"
#include "chns.h"

namespace AMDiS { namespace base_problems {

class CahnHilliardNavierStokes_TwoPhase : public BaseProblem<extensions::ExtendedProblemStat>
{
public: // definition of types

  typedef BaseProblem<extensions::ExtendedProblemStat> super;

public: // public methods

  CahnHilliardNavierStokes_TwoPhase(const std::string &name_);
  ~CahnHilliardNavierStokes_TwoPhase();

  virtual void initData() override;

  virtual void transferInitialSolution(AdaptInfo *adaptInfo) override;  
  virtual void closeTimestep(AdaptInfo *adaptInfo) override;
  
  double getEpsilon() { return eps; }
  int getDoubleWellType() { return doubleWell; }

  DOFVector<WorldVector<double> >* getVelocity()
  {
    return velocity;
  }
  
  virtual void fillOperators() override;
  virtual void fillBoundaryConditions() override {}
  
  virtual void addMaterialDerivativeConst(int i);
  virtual void addMaterialDerivativeNonConst(int i);
  virtual void addLaplaceTermConst(int i);
  virtual void addLaplaceTermNonConst(int i);

protected: // methods
  void calcVelocity()
  {
    if (dow == 1)
      transformDOF(prob->getSolution()->getDOFVector(2), velocity, new AMDiS::Vec1WorldVec<double>);
    else if (dow == 2)
      transformDOF(prob->getSolution()->getDOFVector(2), prob->getSolution()->getDOFVector(3), velocity, new AMDiS::Vec2WorldVec<double>);
    else if (dow == 3)
      transformDOF(prob->getSolution()->getDOFVector(2), prob->getSolution()->getDOFVector(3), prob->getSolution()->getDOFVector(4), velocity, new AMDiS::Vec3WorldVec<double>);
  }

protected: // protected variables

  HL_SignedDistTraverse *reinit;

  DOFVector<WorldVector<double> >* velocity;
  
  bool useMobility;
  bool useConservationForm;

  unsigned dim;
  
  int laplaceType;
  int nonLinTerm;
  int doubleWell;

  // navierStokes parameters
  double oldTimestep;
  double viscosity1;
  double viscosity2;
  double density1;
  double density2;
  double c;
  
  // Cahn-Hilliard parameters
  double gamma;
  double eps;
  double minusEps;
  double epsInv;
  double minusEpsInv;
  double epsSqr;
  double minusEpsSqr;
  
  double sigma;		// coupling parameter to calculate the surface tension
  double surfaceTension;// := sigma/epsilon
  
  WorldVector<double> force;

  FileVectorWriter *fileWriter;
};

} }

#endif // CAHN_HILLIARD_H
