/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef PHASE_FIELD_CRYSTAL_PHASE_H
#define PHASE_FIELD_CRYSTAL_PHASE_H

#include "PhaseFieldCrystal.h"
#include "Expressions.h"

namespace AMDiS { namespace base_problems {

  namespace detail
  {
  
    /** Phase-field Crystal problem in diffuse domain
    */
    template<typename ProblemStatType>
    class PhaseFieldCrystal_Phase : public PhaseFieldCrystal<ProblemStatType>
    {
    public: // typedefs

      typedef PhaseFieldCrystal_Phase<ProblemStatType>  self;
      typedef PhaseFieldCrystal<ProblemStatType>        super;

    public: // methods

      PhaseFieldCrystal_Phase(const std::string &name_);
      
      ~PhaseFieldCrystal_Phase();

      DOFVector<double> *getPhase()
      {
	return phase;
      }

      void setPhase(DOFVector<double>* phase_)
      {
	if (phase == NULL)
	  phase = new DOFVector<double>(self::prob->getFeSpace(1), "phase");
	if (phaseDisturbed == NULL)
	  phaseDisturbed = new DOFVector<double>(self::prob->getFeSpace(1), "phaseDisturbed");

	phase->interpol(phase_);
	*phaseDisturbed << max(valueOf(phase), 1.e-6);
      };

      virtual void fillOperators() override;
      
      double calcEnergy();

    private: // variables

      DOFVector<double> *phase;
      DOFVector<double> *phaseDisturbed;
    };
    
  } // end namespace detail

typedef detail::PhaseFieldCrystal_Phase<AMDiS::ProblemStat> PhaseFieldCrystal_Phase;


} }

#include "PhaseFieldCrystal_Phase.hh"

#endif // PHASE_FIELD_CRYSTAL_PHASE_H
