/******************************************************************************
 *
 * Extension of AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: Simon Praetorius et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/

#ifndef AMDIS_PETSC_PRECON_PFC_H
#define AMDIS_PETSC_PRECON_PFC_H

#include "solver/PetscSolver.h"

namespace AMDiS {

  using namespace std;

  struct PfcData {
    KSP kspM, kspMpK, kspMpK2;
    Mat matM, matK;
    double delta, tau;
  };

  class PetscPreconPfc : public PetscPreconditionerNested
  {
  public:
    typedef PetscMatrixNested    MatrixType;
    typedef PetscVectorNested    VectorType;
    
    typedef PetscPreconditionerNested            super;
    typedef PetscPreconPfc                       self;
    typedef PetscRunner<MatrixType, VectorType>  Runner;
    
    struct Creator : CreatorInterfaceName<super>
    {
      virtual ~Creator() {}
      super* create() { return new self(); }
    };

  public:
    PetscPreconPfc()
      : super("pfc_", "pfc"),
	tau(NULL),
	M0(1.0)
    { }

    void setData(double *tau_, double M0_ = 1.0)
    {
      tau = tau_;
      M0 = M0_;
    }

    void init(PC pc, const SolverMatrix<Matrix<DOFMatrix*> >& A, const MatrixType& fullMatrix) override;
    void exit() override;

  private:
    Mat MpK;
    Mat MpK2;
    PfcData data;

    double *tau;
    double M0;
  };

}

#endif // AMDIS_PETSC_PRECON_PFC_H
